searchNodes=[{"doc":"In-memory key-value store with subscriptions PropertyTable makes it easy to set up a key-value store where users can subscribe to changes based on patterns. PropertyTable refers to keys as properties. Properties have values and are timestamped as to when they received that value. Subscriptions make this library feel similar to Publish-Subscribe. Events, though, are only for changes to properties. PropertyTable is useful when you want to expose a decent amount of state and let consumers pick and choose what parts interest them. PropertyTable consumers express their interest in properties using &quot;patterns&quot;. A pattern could be as simple as the property of interest or it could contain wildcards. This allows one to create hierarchical key-value stores, map-based stores, or just simple key-value stores with notifications. PropertyTable is optionally persistent to disk. Keys and values are backed by ETS.","ref":"PropertyTable.html","title":"PropertyTable","type":"module"},{"doc":"Returns a specification to start a property_table under a supervisor. See Supervisor .","ref":"PropertyTable.html#child_spec/1","title":"PropertyTable.child_spec/1","type":"function"},{"doc":"Delete the specified property","ref":"PropertyTable.html#delete/2","title":"PropertyTable.delete/2","type":"function"},{"doc":"Delete all properties that match a pattern","ref":"PropertyTable.html#delete_matches/2","title":"PropertyTable.delete_matches/2","type":"function"},{"doc":"Fetch a property with the time that it was set Timestamps come from System.monotonic_time()","ref":"PropertyTable.html#fetch_with_timestamp/2","title":"PropertyTable.fetch_with_timestamp/2","type":"function"},{"doc":"If persistence is enabled for this property table, save the current state to disk immediately.","ref":"PropertyTable.html#flush_to_disk/1","title":"PropertyTable.flush_to_disk/1","type":"function"},{"doc":"Get the current value of a property","ref":"PropertyTable.html#get/3","title":"PropertyTable.get/3","type":"function"},{"doc":"Get all properties This function might return a really long list so it's mainly intended for debug or convenience when you know that the table only contains a few properties.","ref":"PropertyTable.html#get_all/1","title":"PropertyTable.get_all/1","type":"function"},{"doc":"Returns a list of availiable snapshot IDs and full name tuples for a property table with persistence enable","ref":"PropertyTable.html#get_snapshots/1","title":"PropertyTable.get_snapshots/1","type":"function"},{"doc":"Get a list of all properties matching the specified property pattern","ref":"PropertyTable.html#match/2","title":"PropertyTable.match/2","type":"function"},{"doc":"Update a property and notify listeners","ref":"PropertyTable.html#put/3","title":"PropertyTable.put/3","type":"function"},{"doc":"Update many properties This is similar to calling put/3 several times in a row, but atomically. It is also slightly more efficient when updating more than one property.","ref":"PropertyTable.html#put_many/2","title":"PropertyTable.put_many/2","type":"function"},{"doc":"If persistence is enabled for this property table, restore the current state of the PropertyTable to that of a past named snapshot","ref":"PropertyTable.html#restore_snapshot/2","title":"PropertyTable.restore_snapshot/2","type":"function"},{"doc":"If persistence is enabled for this property table, save the current state and copy a snapshot of it into the /snapshots sub-directory of the set data directory.","ref":"PropertyTable.html#snapshot/1","title":"PropertyTable.snapshot/1","type":"function"},{"doc":"Start a PropertyTable's supervision tree To create a PropertyTable for your application or library, add the following child_spec to one of your supervision trees: { PropertyTable , name : MyTableName } The :name option is required. All calls to PropertyTable will need to know it and the process will be registered under than name so be sure it's unique. Additional options are: :properties - a list of {property, value} tuples to initially populate the PropertyTable :matcher - set the format for how properties and how they should be matched for triggering events. See PropertyTable.Matcher . :tuple_events - set to true for change events to be in the old tuple format. This is not recommended for new code and hopefully will be removed in the future. Persistent options: You MUST set at least :persist_data_path for any of the other options to be respected! If the table can restore its data from disk, it will IGNORE your initial :properties value. :persist_data_path - set to a directory where PropertyTable will persist the contents of the table to disk, snapshots will also be stored here. :persist_interval - if set PropertyTable will persist the contents of tables to disk in intervals of the provided value (in milliseconds) automatically. :persist_max_snapshots - Maximum number of manual snapshots to keep on disk before they are replaced - (oldest snapshots are replaced first.) Defaults to 25. :persist_compression - 0..9 range to compress the terms when written to disk, see :erlang.term_to_binary/2 . Defaults to 6.","ref":"PropertyTable.html#start_link/1","title":"PropertyTable.start_link/1","type":"function"},{"doc":"Subscribe to receive events","ref":"PropertyTable.html#subscribe/2","title":"PropertyTable.subscribe/2","type":"function"},{"doc":"Stop subscribing to a property","ref":"PropertyTable.html#unsubscribe/2","title":"PropertyTable.unsubscribe/2","type":"function"},{"doc":"PropertyTable configuration options See start_link/2 for usage.","ref":"PropertyTable.html#t:option/0","title":"PropertyTable.option/0","type":"type"},{"doc":"","ref":"PropertyTable.html#t:pattern/0","title":"PropertyTable.pattern/0","type":"type"},{"doc":"Properties","ref":"PropertyTable.html#t:property/0","title":"PropertyTable.property/0","type":"type"},{"doc":"","ref":"PropertyTable.html#t:property_value/0","title":"PropertyTable.property_value/0","type":"type"},{"doc":"A table_id identifies a group of properties","ref":"PropertyTable.html#t:table_id/0","title":"PropertyTable.table_id/0","type":"type"},{"doc":"","ref":"PropertyTable.html#t:value/0","title":"PropertyTable.value/0","type":"type"},{"doc":"Struct sent to subscribers on property changes :table - the table generating this event :property - which property changed :value - the new value :timestamp - the timestamp ( System.monotonic_time/0 ) when the changed happened :previous_value - the previous value ( nil if this property is new) :previous_timestamp - the timestamp when the property changed to :previous_value . Use this to calculate how long the property was the previous value.","ref":"PropertyTable.Event.html","title":"PropertyTable.Event","type":"module"},{"doc":"Convert event to the old tuple event format This is only used for backwards compatibility. At some point, it hopefully will be removed.","ref":"PropertyTable.Event.html#to_tuple/1","title":"PropertyTable.Event.to_tuple/1","type":"function"},{"doc":"","ref":"PropertyTable.Event.html#t:t/0","title":"PropertyTable.Event.t/0","type":"type"},{"doc":"Behaviour for customizing the Matcher logic for filtering and dispatching events","ref":"PropertyTable.Matcher.html","title":"PropertyTable.Matcher","type":"behaviour"},{"doc":"Check whether a pattern is valid Returns :ok on success or {:error, error} where error is an Exception struct with information about the issue.","ref":"PropertyTable.Matcher.html#c:check_pattern/1","title":"PropertyTable.Matcher.check_pattern/1","type":"callback"},{"doc":"Check whether a property is valid Returns :ok on success or {:error, error} where error is an Exception struct with information about the issue.","ref":"PropertyTable.Matcher.html#c:check_property/1","title":"PropertyTable.Matcher.check_property/1","type":"callback"},{"doc":"Returns true if the pattern matches the specified property","ref":"PropertyTable.Matcher.html#c:matches?/2","title":"PropertyTable.Matcher.matches?/2","type":"callback"},{"doc":"Match logic using keys organized as hierarchical lists Property keys that are lists look like [&quot;first&quot;, &quot;second&quot;, &quot;third&quot;] . These are intended to create a hierarchical organization of keys. Matching patterns involves checking whether the pattern is at the beginning of the key. This makes it possible to get notified on every property change where the key begins with [&quot;first&quot;, &quot;second&quot;] . This is a really common use case when using hierarchically organized keys. Two special atoms can be used: :_ - match anything at this part of the list :$ - match the end of the list","ref":"PropertyTable.Matcher.StringPath.html","title":"PropertyTable.Matcher.StringPath","type":"module"},{"doc":"Check whether a pattern is valid Returns :ok on success or {:error, error} where error is an Exception struct with information about the issue.","ref":"PropertyTable.Matcher.StringPath.html#check_pattern/1","title":"PropertyTable.Matcher.StringPath.check_pattern/1","type":"function"},{"doc":"Check whether a property is valid Returns :ok on success or {:error, error} where error is an Exception struct with information about the issue.","ref":"PropertyTable.Matcher.StringPath.html#check_property/1","title":"PropertyTable.Matcher.StringPath.check_property/1","type":"function"},{"doc":"Returns true if the pattern matches the specified property","ref":"PropertyTable.Matcher.StringPath.html#matches?/2","title":"PropertyTable.Matcher.StringPath.matches?/2","type":"function"},{"doc":"This module contains logic to persist the content of a PropertyTable to disk It does so in the following manner: If a prop_table.db file exists in the data directory, rename it to prop_table.db.backup Write the data of the ETS table to the file prop_table.db Delete the backup file When you wish to restore a property table from disk, the following will occur: The stable file will be verified, if it validates, it will be loaded If the stable file failed to validate or did not exist, try and load from the last backup file If the backup file fails, log loudly about data loss You can also manually request &quot;snapshots&quot; of the property table, this will immediately do the procedure for persisting to disk above, then a duplicate of the current prop_table.db file will be copied into the snapshots/ directory with a timestamp added to the file name. max_snapshots in the options keyword list can be used to limit the number of snapshots saved in the snapshots/ directory, if the limit is reached during a snapshot operation, the oldest snapshot will be deleted from disk.","ref":"PropertyTable.Persist.html","title":"PropertyTable.Persist","type":"module"},{"doc":"","ref":"PropertyTable.Persist.html#get_snapshot_list/1","title":"PropertyTable.Persist.get_snapshot_list/1","type":"function"},{"doc":"","ref":"PropertyTable.Persist.html#persist_to_disk/2","title":"PropertyTable.Persist.persist_to_disk/2","type":"function"},{"doc":"","ref":"PropertyTable.Persist.html#restore_from_disk/2","title":"PropertyTable.Persist.restore_from_disk/2","type":"function"},{"doc":"","ref":"PropertyTable.Persist.html#restore_snapshot/2","title":"PropertyTable.Persist.restore_snapshot/2","type":"function"},{"doc":"","ref":"PropertyTable.Persist.html#save_snapshot/2","title":"PropertyTable.Persist.save_snapshot/2","type":"function"},{"doc":"This module contains methods to aid in writing the contents of a PropertyTable to a custom file format. The structure of the format is the following: + -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- + | HEADER &#39;PTABLE&#39; | Used to initially validate the file [ 0x00 ] + -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- + | FILE VERSION ( uint8 ) | Used to track the internal PropertyTable structure of the file [ 0x06 ] + -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- + | RESERVED ( 8 bits ) | Reserved [ 0x07 ] + -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- + | PAYLOAD SIZE ( uint64 ) | Length in bytes of the payload that follows [ 0x08 ] + -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- + | PAYLOAD BYTES ( above size ) | Raw erlang term bytes of the table contents [ 0x10 ] + -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- + | PAYLOAD HASH ( MD5 128 bits ) | MD5 Checksum of the bytes when they were written , ensures table integrity [ 0x10 + payload_size ] + -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- +","ref":"PropertyTable.PersistFile.html","title":"PropertyTable.PersistFile","type":"module"},{"doc":"","ref":"PropertyTable.PersistFile.html#decode_file/1","title":"PropertyTable.PersistFile.decode_file/1","type":"function"},{"doc":"","ref":"PropertyTable.PersistFile.html#encode_binary/1","title":"PropertyTable.PersistFile.encode_binary/1","type":"function"},{"doc":"In-memory key-value store with subscriptions PropertyTable makes it easy to set up a key-value store where users can subscribe to changes based on patterns. PropertyTable refers to keys as properties. Properties have values and are timestamped as to when they received that value. Subscriptions make this library feel similar to Publish-Subscribe. Events, though, are only for changes to properties. PropertyTable is useful when you want to expose a decent amount of state and let consumers pick and choose what parts interest them. PropertyTable consumers express their interest in properties using &quot;patterns&quot;. A pattern could be as simple as the property of interest or it could contain wildcards. This allows one to create hierarchical key-value stores, map-based stores, or just simple key-value stores with notifications. PropertyTable is optionally persistent to disk. Keys and values are backed by ETS.","ref":"readme.html","title":"PropertyTable","type":"extras"},{"doc":"While configurable, the default property style for PropertyTable is a String list. This enables a hierarchical key-value store. One use case that is roughly hierarchical is exposing network interface status to users. Imagine a NetworkTable set up like the following: NetworkTable ├── available_interfaces │ └── [eth0, eth1] └── interface | ├── eth0 | │ ├── config | | | └── %{ipv4: %{method: :dhcp}} | │ └── connection | | └── :internet | └── eth1 | ├── config | | └── %{ipv4: %{method: :static}} | └── connection | └── :disconnected └── connection └── :internet In this example, NetworkTable would be the name of the PropertyTable. The connection status of &quot;eth1&quot; would be represented as [&quot;interface&quot;, &quot;eth1&quot;, &quot;connection&quot;] and have a value of :disconnected . The library maintaining this table (the producer) creates the PropertyTable by adding a child_spec to its supervision tree: { PropertyTable , name : NetworkTable } To run this example from the IEx prompt, start the PropertyTable manually by calling PropertyTable.start_link/1 : PropertyTable . start_link ( name : NetworkTable ) Inserting properties into the table looks like: PropertyTable . put ( NetworkTable , [ &quot;available_interfaces&quot; ] , [ &quot;eth0&quot; , &quot;eth1&quot; ] ) PropertyTable . put ( NetworkTable , [ &quot;connection&quot; ] , :internet ) PropertyTable . put ( NetworkTable , [ &quot;interface&quot; , &quot;eth0&quot; , &quot;config&quot; ] , %{ ipv4 : %{ method : :dhcp } } ) PropertyTable . put ( NetworkTable , [ &quot;interface&quot; , &quot;eth0&quot; , &quot;connection&quot; ] , :internet ) Read one property by running: PropertyTable . get ( NetworkTable , [ &quot;interface&quot; , &quot;eth0&quot; , &quot;config&quot; ] ) Since the format for properties is naturally hierarchical, you can get multiple by matching on a pattern that contains the start of the property that you want: PropertyTable . match ( NetworkTable , [ &quot;interface&quot; ] ) You can subscribe to changes to receive a message after each change happens. For example, to receive a message when any property starting with &quot;interface&quot; changes, run: PropertyTable . subscribe ( table , [ &quot;interface&quot; ] ) Test with: PropertyTable . put ( NetworkTable , [ &quot;interface&quot; , &quot;eth0&quot; , &quot;connection&quot; ] , :disconnected ) flush Then when a property changes value, the Erlang process that called PropertyTable.subscribe/2 will receive a %PropertyTable.Event{} message: % PropertyTable.Event { table : NetworkTable , property : [ &quot;interface&quot; , &quot;eth0&quot; , &quot;connection&quot; ] , value : :disconnected timestamp : 200 , previous_value : :internet , previous_timestamp : 100 } The timestamps in the event are from System.monotonic_time/0 . In this example, you could calculate the time that &quot;eth0&quot; was connected to the internet by subtracting the timestamps.","ref":"readme.html#example","title":"PropertyTable - Example","type":"extras"},{"doc":"The default property format is a list of strings. Patterns are also list of strings and are &quot;prefix&quot; matched. For example, the pattern [&quot;a&quot;] would match the properties [&quot;a&quot;] and [&quot;a&quot;, &quot;b&quot;] but not [&quot;c&quot;] . String path patterns also support two positional wildcards: :&quot;$&quot; - do not match paths that have additional elements :_ - match any string in that location For example, if you want to match [&quot;a&quot;, &quot;b&quot;] exactly, use the pattern [&quot;a&quot;, &quot;b&quot;, :&quot;$&quot;] . Likewise, if you don't care what's an a position in the string, specify :_ like [:_, &quot;b&quot;] .","ref":"readme.html#string-path-properties-and-patterns","title":"PropertyTable - String path properties and patterns","type":"extras"},{"doc":"It's possible to replace the string path pattern matching in PropertyTable by providing an implementation for the PropertyTable.Matcher behaviour. The important function is matches? : @callback matches? ( PropertyTable . pattern ( ) , PropertyTable . property ( ) ) :: boolean ( ) PropertyTable calls this function when deciding whether to send a property change event to a subscriber and when you call PropertyTable.match/2 . Pass your module that implements the PropertyTable.Matcher behaviour as an option to PropertyTable: { PropertyTable , name : NetworkTable , matcher : MyCustomMatcher }","ref":"readme.html#custom-property-types-and-patterns","title":"PropertyTable - Custom property types and patterns","type":"extras"},{"doc":"PropertyTable has a sweet spot in what it supports. It's not intended for very large datasets nor is it the most efficient solution for all patterns. As a rough guide, the use cases we had in mind with PropertyTable have in the low 1000s of keys, a couple producers, a dozen consumers, and changes are bursty. Optimization choices were made with that in mind. This means: Reads query ETS directly, but changes to properties are routed through one GenServer. This reduces processing in the producer's thread context at the cost of creating a potential bottleneck on multicore machines Publishing events iterates over all subscriber patterns. This adds a lot of flexibility to patterns. Given the design target of a dozen or so consumers, it seemed that any indexing or optimization to reduce the number of subscribers looked at would be slower overall than just trying each pattern. Getting a specific property is very fast (one ETS looking on an indexed key), but matching is slow. Matching iterates over every property. This allows for a lot of flexibility in patterns. The expectation that matching is not a common task, and that users will subscribe to changes over repeatedly calling match.","ref":"readme.html#efficiency","title":"PropertyTable - Efficiency","type":"extras"},{"doc":"Copyright (C) 2022 Nerves Project Authors Licensed under the Apache License , Version 2.0 ( the &quot;License&quot; ) ; you may not use this file except in compliance with the License . You may obtain a copy of the License at [ http :// www . apache . org / licenses / LICENSE - 2.0 ] ( http :// www . apache . org / licenses / LICENSE - 2.0 ) Unless required by applicable law or agreed to in writing , software distributed under the License is distributed on an &quot;AS IS&quot; BASIS , WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND , either express or implied . See the License for the specific language governing permissions and limitations under the License .","ref":"readme.html#license","title":"PropertyTable - License","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"New features Automatic persistence and snapshots for PropertyTables. This makes it possible to use PropertyTable for small key/value stores like those for storing runtime settings especially for Nerves devices. PropertyTable protects against corruption and unexpected reboots that happen mid-write.","ref":"changelog.html#v0-2-1","title":"Changelog - v0.2.1","type":"extras"},{"doc":"Backwards incompatible changes nil no longer deletes a property from the table. In other words, it's ok to for properties to be nil now. PropertyTable.clear/2 and PropertyTable.clear_all/2 were renamed to PropertyTable.delete/2 and PropertyTable.delete_matches/2 respectively. PropertyTable.put/3 raises if you give it an invalid property rather than returning an error tuple. Since these are usually programming errors anyway, this change removes the need for a return value check for Dialyzer. New features Added PropertyTable.put_many/2 . It's possible to add many properties to the table atomically. Change notifications still only get sent for properties that really changed.","ref":"changelog.html#v0-2-0","title":"Changelog - v0.2.0","type":"extras"},{"doc":"Initial release This code was extracted from vintage_net , simplified and modified to support more use cases.","ref":"changelog.html#v0-1-0","title":"Changelog - v0.1.0","type":"extras"}]